package question

import (
	"io"
	"nproject.io/gitlab/root/dns/binary"
	"nproject.io/gitlab/root/dns/packet/name"
	"nproject.io/gitlab/root/dns/packet/pointer"
	"nproject.io/gitlab/root/dns/packet/query"
	"strings"
)

type Question struct {
	Name name.Name

	Type  query.QType
	Class query.Class
}

func ParseQuestion(reader io.ReadSeeker) (*Question, error) {
	question := &Question{}
	if nameInstance, err := name.Parse(reader); err == nil {
		question.Name = *nameInstance
	} else {
		return nil, err
	}

	if _type, err := binary.ReadInt16(reader); err == nil {
		question.Type = query.QType(_type)
	} else {
		return nil, err
	}
	if class, err := binary.ReadInt16(reader); err == nil {
		question.Class = query.Class(class)
	} else {
		return nil, err
	}

	return question, nil
}

func (question *Question) String() string {
	buffer := &strings.Builder{}

	buffer.WriteString(question.Name.String())

	return buffer.String()
}

func (question *Question) Write(writer binary.Writer,
	pointerRegister pointer.Register) {
	question.Name.Write(writer,
		pointerRegister)

	_ = binary.WriteInt16(writer, int16(question.Type))
	_ = binary.WriteInt16(writer, int16(question.Class))
}
